# The LearningOnline Network with CAPA
# a pile of common html routines
#
# $Id: lonhtmlcommon.pm,v 1.84 2004/07/26 21:57:27 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
######################################################################
######################################################################

=pod

=head1 NAME

Apache::lonhtmlcommon - routines to do common html things

=head1 SYNOPSIS

Referenced by other mod_perl Apache modules.

=head1 INTRODUCTION

lonhtmlcommon is a collection of subroutines used to present information
in a consistent html format, or provide other functionality related to
html.

=head2 General Subroutines

=over 4

=cut 

######################################################################
######################################################################

package Apache::lonhtmlcommon;

use Time::Local;
use Time::HiRes;
use Apache::lonlocal;
use strict;

##############################################
##############################################

=pod

=item authorbombs

=cut

##############################################
##############################################

sub authorbombs {
    my $url=shift;
    $url=&Apache::lonnet::declutter($url);
    my ($udom,$uname)=($url=~/^(\w+)\/(\w+)\//);
    my %bombs=&Apache::lonmsg::all_url_author_res_msg($uname,$udom);
    foreach (keys %bombs) {
	if ($_=~/^$udom\/$uname\//) {
	    return '<a href="/adm/bombs/'.$url.
		'"><img src="/adm/lonMisc/bomb.gif" border="0" /></a>'.
		&Apache::loncommon::help_open_topic('About_Bombs');
	}
    }
    return '';
}

##############################################
##############################################

sub recent_filename {
    my $area=shift;
    return 'nohist_recent_'.&Apache::lonnet::escape($area);
}

sub store_recent {
    my ($area,$name,$value)=@_;
    my $file=&recent_filename($area);
    my %recent=&Apache::lonnet::dump($file);
    if (scalar(keys(%recent))>10) {
# remove oldest value
	my $oldest=time;
	my $delkey='';
	foreach (keys %recent) {
	    my $thistime=(split(/\&/,$recent{$_}))[0];
	    if ($thistime<$oldest) {
		$oldest=$thistime;
		$delkey=$_;
	    }
	}
	&Apache::lonnet::del($file,[$delkey]);
    }
# store new value
    &Apache::lonnet::put($file,{ $name => 
				 time.'&'.&Apache::lonnet::escape($value) });
}

sub select_recent {
    my ($area,$fieldname,$event)=@_;
    my %recent=&Apache::lonnet::dump(&recent_filename($area));
    my $return="\n<select name='$fieldname'".
	($event?" onChange='$event'":'').
	">\n<option value=''>--- ".&mt('Recent')." ---</option>";
    foreach (sort keys %recent) {
	unless ($_=~/^error\:/) {
	    $return.="\n<option value='$_'>".
		&Apache::lonnet::unescape((split(/\&/,$recent{$_}))[1]).
		'</option>';
	}
    }
    $return.="\n</select>\n";
    return $return;
}


=pod

=item textbox

=cut

##############################################
##############################################
sub textbox {
    my ($name,$value,$size,$special) = @_;
    $size = 40 if (! defined($size));
    my $Str = '<input type="text" name="'.$name.'" size="'.$size.'" '.
        'value="'.$value.'" '.$special.' />';
    return $Str;
}

##############################################
##############################################

=pod

=item checkbox

=cut

##############################################
##############################################
sub checkbox {
    my ($name,$checked,$value) = @_;
    my $Str = '<input type="checkbox" name="'.$name.'" ';
    if (defined($value)) {
        $Str .= 'value="'.$value.'"';
    } 
    if ($checked) {
        $Str .= ' checked="1"';
    }
    $Str .= ' />';
    return $Str;
}

##############################################
##############################################

=pod

=item &date_setter

&date_setter returns html and javascript for a compact date-setting form.
To retrieve values from it, use &get_date_from_form().

Inputs

=over 4

=item $dname 

The name to prepend to the form elements.  
The form elements defined will be dname_year, dname_month, dname_day,
dname_hour, dname_min, and dname_sec.

=item $currentvalue

The current setting for this time parameter.  A unix format time
(time in seconds since the beginning of Jan 1st, 1970, GMT.  
An undefined value is taken to indicate the value is the current time.
Also, to be explicit, a value of 'now' also indicates the current time.

=item $special

Additional html/javascript to be associated with each element in
the date_setter.  See lonparmset for example usage.

=item $includeempty 

=item $state

Specifies the initial state of the form elements.  Either 'disabled' or empty.
Defaults to empty, which indiciates the form elements are not disabled. 

=back

Bugs

The method used to restrict user input will fail in the year 2400.

=cut

##############################################
##############################################
sub date_setter {
    my ($formname,$dname,$currentvalue,$special,$includeempty,$state,
        $no_hh_mm_ss) = @_;
    if (! defined($state) || $state ne 'disabled') {
        $state = '';
    }
    if (! defined($no_hh_mm_ss)) {
        $no_hh_mm_ss = 0;
    }
    if (! defined($currentvalue) || $currentvalue eq 'now') {
	unless ($includeempty) {
	    $currentvalue = time;
	} else {
	    $currentvalue = 0;
	}
    }
    # other potentially useful values:     wkday,yrday,is_daylight_savings
    my ($sec,$min,$hour,$mday,$month,$year)=('','',undef,'','','');
    if ($currentvalue) {
	($sec,$min,$hour,$mday,$month,$year,undef,undef,undef) = 
	    localtime($currentvalue);
	$year += 1900;
    }
    my $result = "\n<!-- $dname date setting form -->\n";
    $result .= <<ENDJS;
<script language="Javascript">
    function $dname\_checkday() {
        var day   = document.$formname.$dname\_day.value;
        var month = document.$formname.$dname\_month.value;
        var year  = document.$formname.$dname\_year.value;
        var valid = true;
        if (day < 1) {
            document.$formname.$dname\_day.value = 1;
        } 
        if (day > 31) {
            document.$formname.$dname\_day.value = 31;
        }
        if ((month == 1)  || (month == 3)  || (month == 5)  ||
            (month == 7)  || (month == 8)  || (month == 10) ||
            (month == 12)) {
            if (day > 31) {
                document.$formname.$dname\_day.value = 31;
                day = 31;
            }
        } else if (month == 2 ) {
            if ((year % 4 == 0) && (year % 100 != 0)) {
                if (day > 29) {
                    document.$formname.$dname\_day.value = 29;
                }
            } else if (day > 29) {
                document.$formname.$dname\_day.value = 28;
            }
        } else if (day > 30) {
            document.$formname.$dname\_day.value = 30;
        }
    }

    function $dname\_disable() {
        document.$formname.$dname\_month.disabled=true;
        document.$formname.$dname\_day.disabled=true;
        document.$formname.$dname\_year.disabled=true;
        document.$formname.$dname\_hour.disabled=true;
        document.$formname.$dname\_minute.disabled=true;
        document.$formname.$dname\_second.disabled=true;
    }

    function $dname\_enable() {
        document.$formname.$dname\_month.disabled=false;
        document.$formname.$dname\_day.disabled=false;
        document.$formname.$dname\_year.disabled=false;
        document.$formname.$dname\_hour.disabled=false;
        document.$formname.$dname\_minute.disabled=false;
        document.$formname.$dname\_second.disabled=false;        
    }

    function $dname\_opencalendar() {
        if (! document.$formname.$dname\_month.disabled) {
            var calwin=window.open(
"/adm/announcements?pickdate=yes&formname=$formname&element=$dname&month="+
document.$formname.$dname\_month.value+"&year="+
document.$formname.$dname\_year.value,
             "LONCAPAcal",
              "height=350,width=350,scrollbars=yes,resizable=yes,menubar=no");
        }

    }
</script>
ENDJS
    $result .= "  <nobr><select name=\"$dname\_month\" ".$special.' '.
        $state.' '.
        "onChange=\"javascript:$dname\_checkday()\" >\n";
    # Month
    my @Months = qw/January February  March     April   May      June 
                    July    August    September October November December/;
    # Pad @Months with a bogus value to make indexing easier
    unshift(@Months,'If you can read this an error occurred');
    if ($includeempty) { $result.="<option value=''></option>"; }
    for(my $m = 1;$m <=$#Months;$m++) {
        $result .= "      <option value=\"$m\" ";
        $result .= "selected " if ($m-1 eq $month);
        $result .= "> ".&mt($Months[$m])." </option>\n";
    }
    $result .= "  </select>\n";
    # Day
    $result .= "  <input type=\"text\" name=\"$dname\_day\" ".$state.' '.
            "value=\"$mday\" size=\"3\" ".$special.' '.
            "onChange=\"javascript:$dname\_checkday()\" />\n";
    # Year
    $result .= "  <input type=\"year\" name=\"$dname\_year\" ".$state.' '.
            "value=\"$year\" size=\"5\" ".$special.' '.
            "onChange=\"javascript:$dname\_checkday()\" />\n";
    $result .= "&nbsp;&nbsp;";
    if (! $no_hh_mm_ss) {
        # Hours
        $result .= "  <select name=\"$dname\_hour\" ".$special." ".$state.' '.
            ">\n";
        if ($includeempty) { $result.="<option value=''></option>"; }
        for (my $h = 0;$h<24;$h++) {
            $result .= "      <option value=\"$h\" ";
            $result .= "selected " if (defined($hour) && $hour == $h);
            $result .= "> ";
            my $timest='';
            if ($h == 0) {
                $timest .= "12 am";
            } elsif($h == 12) {
                $timest .= "12 noon";
            } elsif($h < 12) {
                $timest .= "$h am";
            } else {
                $timest .= $h-12 ." pm";
            }
            $timest=&mt($timest);
            $result .= $timest." </option>\n";
        } 
        $result .= "  </select>\n";
        $result .= "  <input type=\"text\" name=\"$dname\_minute\" ".
            $special.' '.
            $state.' '.
            "value=\"$min\" size=\"3\" /> m\n";
        $result .= "  <input type=\"text\" name=\"$dname\_second\" ".
            $special.' '.
            $state.' '.
            "value=\"$sec\" size=\"3\" /> s\n";
    }
    $result .= "<a href=\"javascript:$dname\_opencalendar()\">".
    &mt('Select Date')."</a></nobr>\n<!-- end $dname date setting form -->\n";
    return $result;
}

##############################################
##############################################

=pod

=item &get_date_from_form

get_date_from_form retrieves the date specified in an &date_setter form.

Inputs:

=over 4

=item $dname

The name passed to &datesetter, which prefixes the form elements.

=item $defaulttime

The unix time to use as the default in case of poor inputs.

=back

Returns: Unix time represented in the form.

=cut

##############################################
##############################################
sub get_date_from_form {
    my ($dname) = @_;
    my ($sec,$min,$hour,$day,$month,$year);
    #
    if (defined($ENV{'form.'.$dname.'_second'})) {
        my $tmpsec = $ENV{'form.'.$dname.'_second'};
        if (($tmpsec =~ /^\d+$/) && ($tmpsec >= 0) && ($tmpsec < 60)) {
            $sec = $tmpsec;
        }
	if (!defined($tmpsec) || $tmpsec eq '') { $sec = 0; }
    } else {
        $sec = 0;
    }
    if (defined($ENV{'form.'.$dname.'_minute'})) {
        my $tmpmin = $ENV{'form.'.$dname.'_minute'};
        if (($tmpmin =~ /^\d+$/) && ($tmpmin >= 0) && ($tmpmin < 60)) {
            $min = $tmpmin;
        }
	if (!defined($tmpmin) || $tmpmin eq '') { $min = 0; }
    } else {
        $min = 0;
    }
    if (defined($ENV{'form.'.$dname.'_hour'})) {
        my $tmphour = $ENV{'form.'.$dname.'_hour'};
        if (($tmphour =~ /^\d+$/) && ($tmphour >= 0) && ($tmphour < 24)) {
            $hour = $tmphour;
        }
    } else {
        $hour = 0;
    }
    if (defined($ENV{'form.'.$dname.'_day'})) {
        my $tmpday = $ENV{'form.'.$dname.'_day'};
        if (($tmpday =~ /^\d+$/) && ($tmpday > 0) && ($tmpday < 32)) {
            $day = $tmpday;
        }
    }
    if (defined($ENV{'form.'.$dname.'_month'})) {
        my $tmpmonth = $ENV{'form.'.$dname.'_month'};
        if (($tmpmonth =~ /^\d+$/) && ($tmpmonth > 0) && ($tmpmonth < 13)) {
            $month = $tmpmonth - 1;
        }
    }
    if (defined($ENV{'form.'.$dname.'_year'})) {
        my $tmpyear = $ENV{'form.'.$dname.'_year'};
        if (($tmpyear =~ /^\d+$/) && ($tmpyear > 1900)) {
            $year = $tmpyear - 1900;
        }
    }
    if (($year<70) || ($year>137)) { return undef; }
    if (defined($sec) && defined($min)   && defined($hour) &&
        defined($day) && defined($month) && defined($year) &&
        eval(&timelocal($sec,$min,$hour,$day,$month,$year))) {
        return &timelocal($sec,$min,$hour,$day,$month,$year);
    } else {
        return undef;
    }
}

##############################################
##############################################

=pod

=item &pjump_javascript_definition()

Returns javascript defining the 'pjump' function, which opens up a
parameter setting wizard.

=cut

##############################################
##############################################
sub pjump_javascript_definition {
    my $Str = <<END;
    function pjump(type,dis,value,marker,ret,call) {
        parmwin=window.open("/adm/rat/parameter.html?type="+escape(type)
                 +"&value="+escape(value)+"&marker="+escape(marker)
                 +"&return="+escape(ret)
                 +"&call="+escape(call)+"&name="+escape(dis),"LONCAPAparms",
                 "height=350,width=350,scrollbars=no,menubar=no");
    }
END
    return $Str;
}

##############################################
##############################################

=pod

=item &javascript_nothing()

Return an appropriate null for the users browser.  This is used
as the first arguement for window.open calls when you want a blank
window that you can then write to.

=cut

##############################################
##############################################
sub javascript_nothing {
    # mozilla and other browsers work with "''", but IE on mac does not.
    my $nothing = "''";
    my $user_browser;
    my $user_os;
    $user_browser = $ENV{'browser.type'} if (exists($ENV{'browser.type'}));
    $user_os      = $ENV{'browser.os'}   if (exists($ENV{'browser.os'}));
    if (! defined($user_browser) || ! defined($user_os)) {
        (undef,$user_browser,undef,undef,undef,$user_os) = 
                           &Apache::loncommon::decode_user_agent();
    }
    if ($user_browser eq 'explorer' && $user_os =~ 'mac') {
        $nothing = "'javascript:void(0);'";
    }
    return $nothing;
}


##############################################
##############################################

=pod

=item &StatusOptions()

Returns html for a selection box which allows the user to choose the
enrollment status of students.  The selection box name is 'Status'.

Inputs:

$status: the currently selected status.  If undefined the value of
$ENV{'form.Status'} is taken.  If that is undefined, a value of 'Active'
is used.

$formname: The name of the form.  If defined the onchange attribute of
the selection box is set to document.$formname.submit().

$size: the size (number of lines) of the selection box.

$onchange: javascript to use when the value is changed.  Enclosed in 
double quotes, ""s, not single quotes.

Returns: a perl string as described.

=cut

##############################################
##############################################
sub StatusOptions {
    my ($status, $formName,$size,$onchange)=@_;
    $size = 1 if (!defined($size));
    if (! defined($status)) {
        $status = 'Active';
        $status = $ENV{'form.Status'} if (exists($ENV{'form.Status'}));
    }

    my $OpSel1 = '';
    my $OpSel2 = '';
    my $OpSel3 = '';

    if($status eq 'Any')         { $OpSel3 = ' selected'; }
    elsif($status eq 'Expired' ) { $OpSel2 = ' selected'; }
    else                         { $OpSel1 = ' selected'; }

    my $Str = '';
    $Str .= '<select name="Status"';
    if(defined($formName) && $formName ne '' && ! defined($onchange)) {
        $Str .= ' onchange="document.'.$formName.'.submit()"';
    }
    if (defined($onchange)) {
        $Str .= ' onchange="'.$onchange.'"';
    }
    $Str .= ' size="'.$size.'" ';
    $Str .= '>'."\n";
    $Str .= '<option value="Active" '.$OpSel1.'>'.
        &mt('Currently Enrolled').'</option>'."\n";
    $Str .= '<option value="Expired" '.$OpSel2.'>'.
        &mt('Previously Enrolled').'</option>'."\n";
    $Str .= '<option value="Any" '.$OpSel3.'>'.
        &mt('Any Enrollment Status').'</option>'."\n";
    $Str .= '</select>'."\n";
}

########################################################
########################################################

=pod

=item Progess Window Handling Routines

These routines handle the creation, update, increment, and closure of 
progress windows.  The progress window reports to the user the number
of items completed and an estimate of the time required to complete the rest.

=over 4


=item &Create_PrgWin

Writes javascript to the client to open a progress window and returns a
data structure used for bookkeeping.

Inputs

=over 4

=item $r Apache request

=item $title The title of the progress window

=item $heading A description (usually 1 line) of the process being initiated.

=item $number_to_do The total number of items being processed.

=item $type Either 'popup' or 'inline' (popup is assumed if nothing is
       specified)

=item $width Specify the width in charaters of the input field.

=item $formname Only useful in the inline case, if a form already exists, this needs to be used and specfiy the name of the form, otherwise the Progress line will be created in a new form of it's own

=item $inputname Only useful in the inline case, if a form and an input of type text exists, use this to specify the name of the input field 

=back

Returns a hash containing the progress state data structure.


=item &Update_PrgWin

Updates the text in the progress indicator.  Does not increment the count.
See &Increment_PrgWin.

Inputs:

=over 4

=item $r Apache request

=item $prog_state Pointer to the data structure returned by &Create_PrgWin

=item $displaystring The string to write to the status indicator

=back

Returns: none


=item Increment_PrgWin

Increment the count of items completed for the progress window by 1.  

Inputs:

=over 4

=item $r Apache request

=item $prog_state Pointer to the data structure returned by Create_PrgWin

=item $extraInfo A description of the items being iterated over.  Typically
'student'.

=back

Returns: none


=item Close_PrgWin

Closes the progress window.

Inputs:

=over 4 

=item $r Apache request

=item $prog_state Pointer to the data structure returned by Create_PrgWin

=back

Returns: none

=back

=cut

########################################################
########################################################

my $uniq=0;
sub get_uniq_name {
    $uniq++;
    return 'uniquename'.$uniq;
}

# Create progress
sub Create_PrgWin {
    my ($r, $title, $heading, $number_to_do,$type,$width,$formname,
	$inputname)=@_;
    if (!defined($type)) { $type='popup'; }
    if (!defined($width)) { $width=55; }
    my %prog_state;
    $prog_state{'type'}=$type;
    if ($type eq 'popup') {
	$prog_state{'window'}='popwin';
	#the whole function called through timeout is due to issues
	#in mozilla Read BUG #2665 if you want to know the whole story
	&r_print($r,'<script>'.
        "var popwin;
         function openpopwin () {
         popwin=open(\'\',\'popwin\',\'width=400,height=100\');".
        "popwin.document.writeln(\'<html><head><title>$title</title></head>".
	      "<body bgcolor=\"#88DDFF\">".
              "<h4>$heading</h4>".
              "<form name=popremain>".
              '<input type="text" size="'.$width.'" name="remaining" value="'.
	      &mt('Starting').'"></form>'.
              "</body></html>\');".
        "popwin.document.close();}".
        "\nwindow.setTimeout(openpopwin,0)</script>");
	$prog_state{'formname'}='popremain';
	$prog_state{'inputname'}="remaining";
    } elsif ($type eq 'inline') {
	$prog_state{'window'}='window';
	if (!$formname) {
	    $prog_state{'formname'}=&get_uniq_name();
	    &r_print($r,'<form name="'.$prog_state{'formname'}.'">');
	} else {
	    $prog_state{'formname'}=$formname;
	}
	if (!$inputname) {
	    $prog_state{'inputname'}=&get_uniq_name();
	    &r_print($r,$heading.' <input type="text" name="'.$prog_state{'inputname'}.
		     '" size="'.$width.'" />');
	} else {
	    $prog_state{'inputname'}=$inputname;
	    
	}
	if (!$formname) { &r_print($r,'</form>'); }
	&Update_PrgWin($r,\%prog_state,&mt('Starting'));
    }

    $prog_state{'done'}=0;
    $prog_state{'firststart'}=&Time::HiRes::time();
    $prog_state{'laststart'}=&Time::HiRes::time();
    $prog_state{'max'}=$number_to_do;
    
    return %prog_state;
}

# update progress
sub Update_PrgWin {
    my ($r,$prog_state,$displayString)=@_;
    &r_print($r,'<script>'.$$prog_state{'window'}.'.document.'.
	     $$prog_state{'formname'}.'.'.
	     $$prog_state{'inputname'}.'.value="'.
	     $displayString.'";</script>');
    $$prog_state{'laststart'}=&Time::HiRes::time();
}

# increment progress state
sub Increment_PrgWin {
    my ($r,$prog_state,$extraInfo)=@_;
    $$prog_state{'done'}++;
    my $time_est= (&Time::HiRes::time() - $$prog_state{'firststart'})/
        $$prog_state{'done'} *
	($$prog_state{'max'}-$$prog_state{'done'});
    $time_est = int($time_est);
    #
    my $min = int($time_est/60);
    my $sec = $time_est % 60;
    # 
    my $str;
    if ($min == 1 && $sec > 1) {
        $str = '[_2] seconds';
    } elsif ($min == 1 && $sec < 2) {
        $str = '1 minute';
    } elsif ($min < 10 && $sec > 1) {
        $str = '[_1] minutes, [_2] seconds';
    } elsif ($min >= 10 || $sec < 2) {
        $str = '[_1] minutes';
    }
    $time_est = &mt($str,$min,$sec);
    #
    my $lasttime = &Time::HiRes::time()-$$prog_state{'laststart'};
    if ($lasttime > 9) {
        $lasttime = int($lasttime);
    } elsif ($lasttime < 0.01) {
        $lasttime = 0;
    } else {
        $lasttime = sprintf("%3.2f",$lasttime);
    }
    if ($lasttime == 1) {
        $lasttime = '('.$lasttime.' '.&mt('second for').' '.$extraInfo.')';
    } else {
        $lasttime = '('.$lasttime.' '.&mt('seconds for').' '.$extraInfo.')';
    }
    #
    my $user_browser = $ENV{'browser.type'} if (exists($ENV{'browser.type'}));
    my $user_os      = $ENV{'browser.os'}   if (exists($ENV{'browser.os'}));
    if (! defined($user_browser) || ! defined($user_os)) {
        (undef,$user_browser,undef,undef,undef,$user_os) = 
                           &Apache::loncommon::decode_user_agent();
    }
    if ($user_browser eq 'explorer' && $user_os =~ 'mac') {
        $lasttime = '';
    }
    &r_print($r,'<script>'.$$prog_state{'window'}.'.document.'.
	     $$prog_state{'formname'}.'.'.
	     $$prog_state{'inputname'}.'.value="'.
	     $$prog_state{'done'}.'/'.$$prog_state{'max'}.
	     ': '.$time_est.' '.&mt('remaining').' '.$lasttime.'";'.'</script>');
    $$prog_state{'laststart'}=&Time::HiRes::time();
}

# close Progress Line
sub Close_PrgWin {
    my ($r,$prog_state)=@_;
    if ($$prog_state{'type'} eq 'popup') {
	&r_print($r,'<script>popwin.close()</script>'."\n");
    } elsif ($$prog_state{'type'} eq 'inline') {
	&Update_PrgWin($r,$prog_state,&mt('Done'));
    }
    undef(%$prog_state);
}

sub r_print {
    my ($r,$to_print)=@_;
    if ($r) {
	$r->print($to_print);
	$r->rflush();
    } else {
	print($to_print);
    }
}

# ------------------------------------------------------- Puts directory header

sub crumbs {
    my ($uri,$target,$prefix,$form,$size,$noformat)=@_;
    if (! defined($size)) {
        $size = '+2';
    }
    my $output='';
    unless ($noformat) { $output.='<br /><tt><b>'; }
    $output.='<font size="'.$size.'">'.$prefix.'/';
    if ($ENV{'user.adv'}) {
	my $path=$prefix.'/';
	foreach (split('/',$uri)) {
	    unless ($_) { next; }
	    $path.=$_;
	    unless ($path eq $uri) { $path.='/'; }
	    my $linkpath=$path;
	    if ($form) {
		$linkpath="javascript:$form.action='$path';$form.submit();";
	    }
	    $output.='<a href="'.$linkpath.'"'.($target?' target="'.$target.'"':'').'>'.$_.'</a>/';
	}
    } else {
	$output.=$uri;
    }
    unless ($uri=~/\/$/) { $output=~s/\/$//; }
    return $output.'</font>'.($noformat?'':'</b></tt><br />');
}

# ------------------------------------------------- Output headers for HTMLArea

sub htmlareaheaders {
    if (&htmlareablocked()) { return ''; }
    unless (&htmlareabrowser()) { return ''; }
    my $lang='en';
    if (&mt('htmlarea_lang') ne 'htmlarea_lang') {
	$lang=&mt('htmlarea_lang');
    }
    return (<<ENDHEADERS);
<script type="text/javascript">
_editor_url='/htmlarea/';
_editor_lang='$lang';
</script>
<script type="text/javascript" src="/htmlarea/htmlarea.js"></script>
ENDHEADERS
}

# ------------------------------------------------- Activate additional buttons

sub htmlareaaddbuttons {
    if (&htmlareablocked()) { return ''; }
    unless (&htmlareabrowser()) { return ''; }
    return (<<ENDADDBUTTON);
    var config=new HTMLArea.Config();
    config.registerButton('ed_math','LaTeX Inline',
			  '/htmlarea/images/ed_math.gif',false,
			    function(editor,id) {
			      editor.surroundHTML('<m>\$','\$</m>');
			    }
			  );
    config.registerButton('ed_math_eqn','LaTeX Equation',
			  '/htmlarea/images/ed_math_eqn.gif',false,
			    function(editor,id) {
			      editor.surroundHTML(
				     '<center><m>\\\\[','\\\\]</m></center>');
			    }
			  );
    config.toolbar.push(['ed_math','ed_math_eqn']);
ENDADDBUTTON
}

# ----------------------------------------------------------------- Preferences

sub disablelink {
    my @fields=@_;
    if (defined($#fields)) {
	unless ($#fields>=0) { return ''; }
    }
    return '<a href="/adm/preferences?action=set_wysiwyg&wysiwyg=off&returnurl='.&Apache::lonnet::escape($ENV{'REQUEST_URI'}).'">'.&mt('Disable WYSIWYG Editor').'</a>';
}

sub enablelink {
    my @fields=@_;
    if (defined($#fields)) {
	unless ($#fields>=0) { return ''; }
    }
    return '<a href="/adm/preferences?action=set_wysiwyg&wysiwyg=on&returnurl='.&Apache::lonnet::escape($ENV{'REQUEST_URI'}).'">'.&mt('Enable WYSIWYG Editor').'</a>';
}

# ----------------------------------------- Script to activate only some fields

sub htmlareaselectactive {
    my @fields=@_;
    unless (&htmlareabrowser()) { return ''; }
    if (&htmlareablocked()) { return '<br />'.&enablelink(@fields); }
    my $output='<script type="text/javascript" defer="1">'.
	&htmlareaaddbuttons();
    foreach(@fields) {
	$output.="\nHTMLArea.replace('$_',config);";
    }
    $output.="\nwindow.status='Activated Editfields';\n</script><br />".
	&disablelink(@fields);
    return $output;
}

# --------------------------------------------------------------------- Blocked

sub htmlareablocked {
    unless ($ENV{'environment.wysiwygeditor'} eq 'on') { return 1; }
    return 0;
}

# ---------------------------------------- Browser capable of running HTMLArea?

sub htmlareabrowser {
    return 1;
}

############################################################
############################################################

=pod

=item breadcrumbs

Compiles the previously registered breadcrumbs into an series of links.
FAQ and BUG links will be placed on the left side of the table if they
are defined for the last registered breadcrumb.  
Additionally supports a 'component', which will be displayed on the
right side of the table (without a link).
A link to help for the component will be included if one is specified.

All inputs can be undef without problems.

Inputs: $color (the background color of the table returned),
        $component (the large text on the right side of the table),
        $component_help
        $function (role to get colors from)
        $domain   (domian of role)
        $menulink (boolean, controls whether to include a link to /adm/menu)

Returns a string containing breadcrumbs for the current page.

=item clear_breadcrumbs

Clears the previously stored breadcrumbs.

=item add_breadcrumb

Pushes a breadcrumb on the stack of crumbs.

input: $breadcrumb, a hash reference.  The keys 'href','title', and 'text'
are required.  If present the keys 'faq' and 'bug' will be used to provide
links to the FAQ and bug sites.

returns: nothing    

=cut

############################################################
############################################################
{
    my @Crumbs;
    
    sub breadcrumbs {
        my ($color,$component,$component_help,$function,$domain,$menulink) =
	    @_;
        if (! defined($color)) {
            if (! defined($function)) {
                $function = &Apache::loncommon::get_users_function();
            }
            $color = &Apache::loncommon::designparm($function.'.tabbg',
                                                    $domain);
        }
        #
        my $Str = "\n".
            '<table width="100%" border="0" cellpadding="0" cellspacing="0">'.
            '<tr><td bgcolor="'.$color.'">'.
            '<font size="-1">';
        #
        # Make the faq and bug data cascade
        my $faq = '';
        my $bug = '';
        # The last breadcrumb does not have a link, so handle it separately.
        my $last = pop(@Crumbs);
        #
        # The first one should be the course or a menu link
	if (!defined($menulink)) { $menulink=1; }
        if ($menulink) {
            my $description = 'Menu';
            if (exists($ENV{'request.course.id'}) && 
                $ENV{'request.course.id'} ne '') {
                $description = 
                    $ENV{'course.'.$ENV{'request.course.id'}.'.description'};
            }
            unshift(@Crumbs,{
                    href   =>'/adm/menu',
                    title  =>'Go to main menu',
                    target =>'_top',
                    text   =>$description,
                });
        }
        my $links .= 
            join('-&gt;',
                 map {
                     $faq = $_->{'faq'} if (exists($_->{'faq'}));
                     $bug = $_->{'bug'} if (exists($_->{'bug'}));
                     my $result = '<a href="'.$_->{'href'}.'" ';
                     if (defined($_->{'target'}) && $_->{'target'} ne '') {
                         $result .= 'target="'.$_->{'target'}.'" ';
                     }
                     $result .='title="'.&mt($_->{'title'}).'">'.
                         &mt($_->{'text'}).'</a>';
                     $result;
                     } @Crumbs
                 );
        $links .= '-&gt;' if ($links ne '');
        $links .= '<b>'.&mt($last->{'text'}).'</b>';
        #
        my $icons = '';
        $faq = $last->{'faq'} if (exists($last->{'faq'}));
        $bug = $last->{'bug'} if (exists($last->{'bug'}));
#        if ($faq ne '') {
#            $icons .= &Apache::loncommon::help_open_faq($faq);
#        }
#        if ($bug ne '') {
#            $icons .= &Apache::loncommon::help_open_bug($bug);
#        }
        $icons .= &Apache::loncommon::help_open_menu($color,$component,$component_help,$function,$faq,$bug);
        if ($icons ne '') {
            $Str .= $icons.'&nbsp;';
        }
        #
        $Str .= $links.'</font></td>';
        #
        if (defined($component)) {
            $Str .= '<td align="right" bgcolor="'.$color.'">'.
                '<font size="+1">'.&mt($component).'</font></td>';
        }
        $Str .= '</tr></table>'."\n";
        #
        # Return the @Crumbs stack to what we started with
        push(@Crumbs,$last);
        shift(@Crumbs);
        #
        return $Str;
    }

    sub clear_breadcrumbs {
        undef(@Crumbs);
    }

    sub add_breadcrumb {
        push (@Crumbs,@_);
    }

} # End of scope for @Crumbs

############################################################
############################################################


1;

__END__
