# The LearningOnline Network with CAPA
# Construction Space Directory Lister
#
# $Id: lonpubdir.pm,v 1.62 2004/07/02 09:41:07 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###

package Apache::lonpubdir;

use strict;
use Apache::File;
use File::Copy;
use Apache::Constants qw(:common :http :methods);
use Apache::loncacc;
use Apache::loncommon();
use Apache::lonhtmlcommon();
use Apache::lonlocal;
use Apache::lonmsg;

sub handler {

  my $r=shift;

  my $fn;



  $fn = getEffectiveUrl($r);

  # Validate access to the construction space and get username@domain.

  my $uname;
  my $udom;

  ($uname,$udom)=
    &Apache::loncacc::constructaccess(
             $fn,$r->dir_config('lonDefDomain')); 
  unless (($uname) && ($udom)) {
     $r->log_reason($uname.' at '.$udom.
         ' trying to list directory '.$ENV{'form.filename'}.
         ' ('.$fn.') - not authorized', 
         $r->filename); 
     return HTTP_NOT_ACCEPTABLE;
  }

  # Remove trailing / from directory name.

  $fn=~s/\/$//;

  unless ($fn) { 
     $r->log_reason($ENV{'user.name'}.' at '.$ENV{'user.domain'}.
         ' trying to list empty directory', $r->filename); 
     return HTTP_NOT_FOUND;
  } 

# ----------------------------------------------------------- Start page output

  my $thisdisfn=$fn;
  $thisdisfn=~s/^\/home\/$uname\/public_html//;	# subdirectory part of
                                                # construction space. 
  my $docroot=$r->dir_config('lonDocRoot');     # Apache  londocument root.

  my $resdir=$docroot.'/res/'.$udom.'/'.$uname.$thisdisfn; # Resource directory
  my $targetdir=$udom.'/'.$uname.$thisdisfn; # Publiction target directory.
  my $linkdir='/priv/'.$uname.$thisdisfn;      # Full URL name of constr space.

  my %bombs=&Apache::lonmsg::all_url_author_res_msg($uname,$udom);

  &startpage($r, $uname, $udom, $thisdisfn);   # Put out the start of page.
  
  # Start off the directory table.

  $r->print('<table border="0" cellspacing="2" cellpadding="2">'.
	    '<tr><th>&nbsp;</th><th>'.&mt('Actions').'</th><th>'.&mt('Name').'</th><th>'.
	    &mt('Title').'</th>'.
	    '<th>'.&mt('Status').'</th><th>'.&mt('Last Modified').
	    '</th></tr>');

  my $filename;
  my $dirptr=16384;		# Mask indicating a directory in stat.cmode.

  opendir(DIR,$fn);
  my @files=sort {uc($a) cmp uc($b)} (readdir(DIR));
  foreach my $filename (@files) {
     my ($cdev,$cino,$cmode,$cnlink,
         $cuid,$cgid,$crdev,$csize,
         $catime,$cmtime,$cctime,
         $cblksize,$cblocks)=stat($fn.'/'.$filename);

     my $extension='';
     if ($filename=~/\.(\w+)$/) { $extension=$1; }
     if ($cmode&$dirptr) {
	 putdirectory($r, $thisdisfn, $linkdir, $filename, $cmtime,$targetdir,\%bombs);
     } elsif (&Apache::loncommon::fileembstyle($extension) ne 'hdn') {
	 putresource($r, $uname, $filename, $thisdisfn, $resdir, 
		     $targetdir, $linkdir, $cmtime,\%bombs);
     } else {
	# "hidden" extension and not a directory, so hide it away.
     }
  }
  closedir(DIR);

  $r->print('</table></body></html>');
  return OK;  
}
#
#  Gets the effective URL of the request and returns it:
#    $effn = getEffectiveUrl($r);
#       $r  - The Apache Request object.
sub getEffectiveUrl {
    my $r = shift;
    my $fn;
    
    if ($ENV{'form.filename'}) {	# If a form filename is defined.
	$fn=$ENV{'form.filename'};
	#
	#   Replace the ~username of the URL with /home/username/public_html
	#   so that we don't have to worry about ~ expansion internally.
	#
	$fn=~s/^http\:\/\/[^\/]+\///;
        $fn=~s/^\///;
        $fn=~s/\~(\w+)/\/home\/$1\/public_html/;
	
	#  Remove trailing / strings (?) 
	
	$fn=~s/\/[^\/]+$//;
    } else {
	#   If no form is defined, use request filename.
	$fn = $r->filename();
	my $lonDocRoot=$r->dir_config('lonDocRoot');
	if ( $fn =~ /$lonDocRoot/ ) {
	    #internal authentication, needs fixup.
	    $fn = $r->uri(); # non users do not get the full path request
                             # through SCRIPT_FILENAME
	    $fn=~s|^/~(\w+)|/home/$1/public_html|;
	}
    }
    $fn=~s/\/+/\//g;
    return $fn;
}
#
#   Output the header of the page.  This includes:
#   - The HTML header 
#   - The H1/H3  stuff which includes the directory.
#
#     startpage($r, $uame, $udom, $thisdisfn);
#      $r     - The apache request object.
#      $uname - User name.
#      $udom  - Domain name the user is logged in under.
#      $thisdisfn - Displayable version of the filename.

sub startpage {
    my ($r, $uname, $udom, $thisdisfn) = @_;
    
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    
    $r->print('<html><head><title>LON-CAPA Construction Space</title></head>');
    
    $r->print(&Apache::loncommon::bodytag(undef,undef,undef,1));
    my $pubdirscript=(<<ENDPUBDIRSCRIPT);
<script>
// Store directory location for menu bar to find

parent.lastknownpriv='/~$uname/$thisdisfn/';

// Confirmation dialogues

    function pubdir(theform) {
	if (confirm('Publish complete directory?')) {
	    theform.submit();
        }
    }
   function pubrecdir(theform) {
	if (confirm('Publish directory and all subdirectories?')) {
            theform.pubrec.value='1';
	    theform.submit();
        }
    }
</script>
ENDPUBDIRSCRIPT

    $r->print('<h2>'.&mt('Construction Space Directory').'</h2>'.
	      '<script type="text/javascript">top.document.title = \''.
	      $thisdisfn.'/ - LON-CAPA Construction Space\';</script>'.
	      $pubdirscript.
              '<form method="post" action="/adm/publish" target="_parent">'.
              '<table><tr><td><input type="hidden" name="filename" value="/~'.
               $uname.$thisdisfn.'/" />'.
              '<input type="button" onClick="pubdir(this.form);" value="'.
&mt('Publish Directory').'" />'.
              '<input type="hidden" name="pubrec" value="" />'.
              '<input type="button" onClick="pubrecdir(this.form);" value="'.
&mt('Publish Directory and Sub Directories').'" /></td><td>'.
'<input type="button" onClick="window.location='."'/~".
               $uname.$thisdisfn."/default.meta'".'" value="'.
&mt('Edit Directory Catalog Information').'" /></td></tr><tr><td><input type="checkbox" name="forcerepub" /> '.&mt('Force publication of unmodified files').'.</td><td>&nbsp;</td></tr></table></form>');
    
    if (($uname ne $ENV{'user.name'}) || ($udom ne $ENV{'user.domain'})) {
	$r->print('<h3>'.&mt('Co-Author').': '.$uname.' at '.$udom.
		  '</h3>');
    }
    my $formaction='/priv/'.$uname.'/'.$thisdisfn.'/';
    $formaction=~s/\/+/\//g;
    $r->print('<form name="dirs" method="post" action="'.$formaction
	      .'" target="_top">'.
      &Apache::lonhtmlcommon::crumbs($uname.'/'.$thisdisfn.'/','_top','/priv').
      &Apache::lonhtmlcommon::select_recent('construct','recent',
	         'this.form.action=this.form.recent.value;this.form.submit()').
	      '</form>');
    &Apache::lonhtmlcommon::store_recent('construct',$formaction,$formaction);
}

#
#   Get the title string or "[untitled]" if the file has no title metadata:
#   Without the latter substitution, it's impossible to examine metadata for
#   untitled resources.  Resources may be legitimately untitled, to prevent
#   searches from locating them.
#
#   $str = getTitleString($fullname);
#       $fullname - Fully qualified filename to check.
#
sub getTitleString {
    my $fullname = shift;
    my $title    = &Apache::lonnet::metadata($fullname, 'title');

    unless ($title) {
	$title = "[".&mt('untitled')."]";
    }
    return $title;
}

sub getCopyRightString {
    my $fullname = shift;
    return &Apache::lonnet::metadata($fullname, 'copyright');
}

sub getSourceRightString {
    my $fullname = shift;
    return &Apache::lonnet::metadata($fullname, 'sourceavail');
}
#
#  Put out a directory table row:
#    putdirectory(r, base, here, dirname, modtime)
#      r       - Apache request object.
#      reqfile - File in request.
#      here    - Where we are in directory tree.
#      dirname - Name of directory special file.
#      modtime - Encoded modification time.
# 
sub putdirectory {
    my ($r, $reqfile, $here, $dirname, $modtime, $resdir, $bombs) = @_;
  
    # construct the display filename: the directory name unless ..:
    
    my $disfilename = $dirname;
    if ($dirname eq '..') {
	$disfilename = '<i>'.&mt('Parent Directory').'</i>';
    }
    unless (( ($dirname eq '..') && ($reqfile eq '')) ||
	    ($dirname eq '.')) {
	my $kaputt=0;
	foreach (keys %{$bombs}) {
	    if ($_=~m:^\Q$resdir\E/\Q$disfilename\E/:) { $kaputt=1; last; }
	}
	%Apache::lonpublisher::metadatafields=();
	%Apache::lonpublisher::metadatakeys=();
	my $construct=$here;
	$construct=~s:^/priv/(\w+)$:/home/$1/public_html:;
	&Apache::lonpublisher::metaeval(&Apache::lonnet::getfile(
       				 $construct.'/'.$dirname.'/default.meta'
								 ));
	$r->print('<tr bgcolor="#CCCCFF">'.
		  '<td><img src="'.
		  $Apache::lonnet::perlvar{'lonIconsURL'}.'/folder_closed.gif" /></td>'.
		  '<td>'.&mt('Go to ...').'</td>'.
		  '<td><font face="arial"><a href="'.$here.'/'.$dirname.'/" target="_top">'.
		  $disfilename.'</a></font></td>'.
		        '<td colspan="2">'.($kaputt?&Apache::lonhtmlcommon::authorbombs($resdir.'/'.$disfilename.'/'):'').$Apache::lonpublisher::metadatafields{'title'}.' <i>'.
		  $Apache::lonpublisher::metadatafields{'subject'}.'</i> '.
		  $Apache::lonpublisher::metadatafields{'keywords'}.'</td>'.
		  '<td>'.&Apache::lonlocal::locallocaltime($modtime).'</td>'.
		  "</tr>\n");
    }	
    return OK;
}
#
#   Put a table row for a file resource.
#
sub putresource {
    my ($r, $uname, $filename, $thisdisfn, 
	$resdir, $targetdir, $linkdir,
	$cmtime,$bombs) = @_;

    my $status=&mt('Unpublished');
    my $bgcolor='#FFAA99';
    my $title='&nbsp;';
    my $publish_button=&mt('Publish');
    if (-e $resdir.'/'.$filename) {
	my ($rdev,$rino,$rmode,$rnlink,
	    $ruid,$rgid,$rrdev,$rsize,
	    $ratime,$rmtime,$rctime,
	    $rblksize,$rblocks)=stat($resdir.'/'.$filename);
	$publish_button=&mt('Re-Publish');
	if ($rmtime>=$cmtime) {
	    $status=&mt('Published').'<br />'.
		&mt(&getCopyRightString($targetdir.'/'.$filename)).' '.
		&mt(&getSourceRightString($targetdir.'/'.$filename));
	    $bgcolor='#CCFF88';
	    if (&Apache::lonnet::metadata($targetdir.'/'.$filename,'obsolete')) {
		$status=&mt('Obsolete');
                $bgcolor='#AAAAAA';
	    }
	    $title='<a href="/res/'.$targetdir.'/'.$filename.
		'.meta" target=cat>'.
		&getTitleString($targetdir.'/'.$filename).'</a>';
	} else {
	    $status=&mt('Modified').'<br />'.
		&mt(&getCopyRightString($targetdir.'/'.$filename)).' '.
		&mt(&getSourceRightString($targetdir.'/'.$filename));
            $bgcolor='#FFFF77';
	    $title='<a href="/res/'.$targetdir.'/'.$filename.'.meta" target=cat>'.
		&getTitleString($targetdir.'/'.$filename).'</a>';
	    if (&Apache::loncommon::fileembstyle(($filename=~/\.(\w+)$/)) eq 'ssi') {
		$status.='<br /><a href="/adm/diff?filename=/~'.$uname.
		    $thisdisfn.'/'.$filename.
		    '&versiontwo=priv" target=cat>'.&mt('Diffs').'</a>';
	    }
	}
	$title.='<br /><a href="/~'.$uname.$thisdisfn.'/'.$filename.'.meta">'. 
	    ($$bombs{$targetdir.'/'.$filename}?'<img src="/adm/lonMisc/bomb.gif" border="0" />':'Edit Metadata').'</a>';
	$status.='<br><a href="/adm/retrieve?filename=/~'.$uname.
	    $thisdisfn.'/'.$filename.'" target="_top">'.&mt('Retrieve').'</a>';
    }
    my $editlink='';
    my $editlink2='';
    if ($filename=~/\.(xml|html|htm|xhtml|xhtm|sty)$/) {
	$editlink=' <br />(<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_top">'.&mt('Edit').'</a>)';
    }
    if ($filename=~/\.(problem|exam|quiz|assess|survey|form|library)$/) {
	$editlink=' (<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_top">'.&mt('EditXML').'</a>)';
	$editlink2=' <br />(<a href="'.$linkdir.'/'.$filename.'?forceColoredit=1" target="_top">'.&mt('Edit').'</a>)';
    }
    if ($filename=~/\.(zip|tar|bz2|gz|tar.gz|tar.bz2|tgz)$/) {
	$editlink=' (<a target="_parent" href="/adm/cfile?decompress=/~'.
	      $uname.$thisdisfn.'/'.$filename.'">'.&mt('Decompress').'</a>)';
    }
    $r->print('<tr bgcolor="'.$bgcolor.'">'.
	      '<td>'.($filename=~/[\#\~]$/?'&nbsp;':
		      '<img src="'.&Apache::loncommon::icon($filename).'" /></td>').
	      '<td><a target="_parent" href="/adm/publish?filename=/~'.
	      $uname.$thisdisfn.'/'.$filename.'">'.$publish_button.'</a>'.
	      '</td>'.
	      '<td><font face="arial">'.
	      '<a href="'.$linkdir.'/'.$filename.'" target="_top">'.
               $filename.'</a></font>'.$editlink2.$editlink.
	      '</td>'.
	      '<td>'.$title.'</td>'.
	      '<td>'.$status.'</td>'.
	      '<td>'.&Apache::lonlocal::locallocaltime($cmtime).'</td>'.
	      "</tr>\n");
    return OK;
}
#
#   Categorize files in the directory.
#   For each file in a list of files in a file directory, 
#   the  file categorized as one of:
#    - directory  
#    - sequence
#    - problem 
#    - Other resource.
#
#   For each file the modification date is determined as well.
#   Returned is a list of sublists:
#    (directories, sequences, problems, other)
#   each of the sublists contains entries of the following form (sorted by
#   filename):
#     (filename, typecode, lastmodtime)
#
#   $list = CategorizeFiles($location, $files)
#       $location   - Directory in which the files live (relative to our
#                     execution.
#       $files      - list of files.
#
sub CategorizeFiles {
    my $location = shift;
    my $files    = shift;
}

1;
__END__

=head1 NAME

Apache::lonpubdir - Construction space directory lister

=head1 SYNOPSIS

Invoked (for various locations) by /etc/httpd/conf/srm.conf:

 <LocationMatch "^/\~.*/$">
 PerlAccessHandler       Apache::loncacc
 SetHandler perl-script
 PerlHandler Apache::lonpubdir
 ErrorDocument     403 /adm/login
 ErrorDocument     404 /adm/notfound.html
 ErrorDocument     406 /adm/unauthorized.html
 ErrorDocument	  500 /adm/errorhandler
 </LocationMatch>

 <Location /adm/pubdir>
 PerlAccessHandler       Apache::lonacc
 SetHandler perl-script
 PerlHandler Apache::lonpubdir
 ErrorDocument     403 /adm/login
 ErrorDocument     404 /adm/notfound.html
 ErrorDocument     406 /adm/unauthorized.html
 ErrorDocument	  500 /adm/errorhandler
 </Location>

=head1 INTRODUCTION

This module publishes a directory of files.

This is part of the LearningOnline Network with CAPA project
described at http://www.lon-capa.org.

=head1 HANDLER SUBROUTINE

This routine is called by Apache and mod_perl.

=over 4

=item *

read in information

=item *

start page output

=item *

run through list of files and attempt to publish unhidden files

=back

=cut
